C	THIS IS A COLLECTION OF THE FOLLOWING SUBROUTINES;

C      ________________________________
C      MODIFIED BY HALDUN O. TEZEL 1986
C      --------------------------------


C		WASPT1
C		WASPV1
C		WASHP1
C		WASUV1
C		WASPS1
C               WASPU1
C		WASHV1
C		WASPR1
C		WASR21
C
C
	SUBROUTINE WASPT1 (VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L)

C  THIS ROUTINE EVALUATES THE WATER/VAPOUR PROPERTIES AS A FUNCTION
C  OF PRESSURE AND TEMPERATURE

C  	VI1	PRESSURE
C	VI2	TEMPERATURE
C	VO1	TEMPERATURE
C	VO2	SPECIFIC VOLUME      GIVE AN APPROXIMATE VALUE
C	L	NUMBER OF ITERATIONS AND IF EITHER TNEW OR VNEW IS
C		LESS THAN ZERO WASPR1 CALLS WASIN1 WHICH EVALUATES
C		LIQUID (L=100) OR VAPOUR (L=200) PROPERTIES.

C	IN THE FINAL RETURN               Tnew WILL BE IN VO1
C	FROM THIS SUBROUTINE              Vnew WILL BE IN VO2

	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
	COMMON /CONST5/ ZZZ0,ZZZ1,ZZZ2,ZZZ3,ZZZ4,ZZZ5,ZZZ6,ZZZ7,ZZZ8,
     *	ZZZ9,ZZZE2,ZZZE3,ZZZE4,ZZZE5,ZZZE6,ZZZM2,ZZZM3,ZZZM4,ZZZM5,
     *	ZZZM6,ZZZ01,ZZZ10,ZZZ273,ZZZ314,ZZZ981,ZZZER

	COMMON /WASPR5/ F1,F2,F11,F12,F22,F21,TES1,TES2,TNEW,VNEW,TOLD,
     *                  VOLD,POLD,IS2

	POLD=VI1
	TOLD=VI2
	VO1=VI2
	VNEW=VO2
	IS2=3
	TES1=ZZZM4
	TES2=ZZZ1
	TNEW=TOLD
	F11=ZZZ1
	F2=ZZZ0
	F21=ZZZ1
	F22=ZZZ0
C
	CALL WASPR1(1,VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L,IRET)
C
 1	F1=Z2+VI1
	F12=+Z22
C
	CALL WASPR1(3,VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L,IRET)	
C
	IF (IRET.GT.ZZZ0) GOTO 1
	RETURN
	END
C
C==============================================================
C
	SUBROUTINE WASPV1 (VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L)
C
C	VI1	P, Pa
C	VI2	V, m3/kg
C	VO1	T, deg.C
C	VO2	V, m3/kg
C
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
	COMMON /CONST5/ ZZZ0,ZZZ1,ZZZ2,ZZZ3,ZZZ4,ZZZ5,ZZZ6,ZZZ7,ZZZ8,
     *	ZZZ9,ZZZE2,ZZZE3,ZZZE4,ZZZE5,ZZZE6,ZZZM2,ZZZM3,ZZZM4,ZZZM5,
     *	ZZZM6,ZZZ01,ZZZ10,ZZZ273,ZZZ314,ZZZ981,ZZZER
C
	COMMON /WASPR5/ F1,F2,F11,F12,F22,F21,TES1,TES2,TNEW,VNEW,TOLD,
     *                  VOLD,POLD,IS2
C
C
	POLD=VI1
	VOLD=VI2
	TNEW=VO1
	VO2=VI2
C
	IS2=6
C
	NX=L
	VNEW=VOLD
	TES1=ZZZER
	TES2=ZZZ1
	F12=ZZZ1
	F2=ZZZ0
	F21=ZZZ0
	F22=ZZZ1
C
	CALL WASPR1 (1,VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L,IRET)
C
 1	F1=-Z2-VI1
	F11=-Z12
C
	CALL WASPR1 (3,VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L,IRET)
C
	IF (IRET.GT.ZZZ0) GOTO 1
C
 99	RETURN
	END
C
C================================================================
C
	SUBROUTINE WASHP1 (VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L)
C
C  THIS SUBROUTINE COMPUTES THE HELMHOLTZ FREE ENERGY FOR SINGLE
C  PHASE (LIG. OR VAPOUR) AS A FUNCTION OF ENTHALPY AND PRESSURE
C
C  PARAMETERS:
C  L	Number of iterations required for solution
C  VI1	Enthalpy, J/Kg
C  VI2	Pressure, Pa
C  VO1	Temperature, Deg.C
C  VO2	Specific volume, m3/Kg
C
C
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
	COMMON /CONST5/ ZZZ0,ZZZ1,ZZZ2,ZZZ3,ZZZ4,ZZZ5,ZZZ6,ZZZ7,ZZZ8,
     *	ZZZ9,ZZZE2,ZZZE3,ZZZE4,ZZZE5,ZZZE6,ZZZM2,ZZZM3,ZZZM4,ZZZM5,
     *	ZZZM6,ZZZ01,ZZZ10,ZZZ273,ZZZ314,ZZZ981,ZZZER
C
	COMMON /WASPR5/ F1,F2,F11,F12,F22,F21,TES1,TES2,TNEW,VNEW,TOLD,
     *                  VOLD,POLD,IS2
C
	POLD=VI2
	TNEW=VO1
	VNEW=VO2
	IS2=4
	TES1=ZZZER
	TES2=ZZZER
C
	CALL WASPR1 (1,VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L,IRET)
C
  1	T=(TOLD+ZZZ273)
C
	F1=(Z-T*Z1-VOLD*Z2)-VI1
	F2=-(Z2+VI2)
	F11=-(T*Z11+VOLD*Z12)
	F12=-(T*Z12+VOLD*Z22)
	F21=-Z12
	F22=-Z22
C
	CALL WASPR1 (3,VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L,IRET)
C
	IF (IRET.GT.0) GOTO 1
	RETURN
	END
C
C================================================================
C
	SUBROUTINE WASUV1 (VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L)
C
C	VI1	U
C	VI2	V
C	VO1	T
C	VO2	V
C
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
	COMMON /CONST5/ ZZZ0,ZZZ1,ZZZ2,ZZZ3,ZZZ4,ZZZ5,ZZZ6,ZZZ7,ZZZ8,
     *	ZZZ9,ZZZE2,ZZZE3,ZZZE4,ZZZE5,ZZZE6,ZZZM2,ZZZM3,ZZZM4,ZZZM5,
     *	ZZZM6,ZZZ01,ZZZ10,ZZZ273,ZZZ314,ZZZ981,ZZZER
C
	COMMON /WASPR5/ F1,F2,F11,F12,F22,F21,TES1,TES2,TNEW,VNEW,TOLD,
     *                  VOLD,POLD,IS2
C
	VOLD=VI2
	VNEW=VOLD
	TNEW=VO1
	TOLD=TNEW
C	
	IS2=1
C
	TES1=ZZZM5
	TES2=ZZZ1
C
	F12=ZZZ1
	F2=ZZZ0
	F21=ZZZ0
	F22=ZZZ1
C
	CALL WASPR1 (1,VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L,IRET)
C
 1	T=(TOLD+ZZZ273)
	F1=(Z-T*Z1)-VI1
	F11=-T*Z11
C
	CALL WASPR1 (3,VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L,IRET)	
C
	IF (IRET.GT.ZZZ0) GOTO 1
 99	RETURN
	END
C
C================================================================
C
	SUBROUTINE WASPS1 (VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L)
C
C	VI1	P
C	VI2	S
C	VO1	T
C	VO2	V
C
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
	COMMON /CONST5/ ZZZ0,ZZZ1,ZZZ2,ZZZ3,ZZZ4,ZZZ5,ZZZ6,ZZZ7,ZZZ8,
     *	ZZZ9,ZZZE2,ZZZE3,ZZZE4,ZZZE5,ZZZE6,ZZZM2,ZZZM3,ZZZM4,ZZZM5,
     *	ZZZM6,ZZZ01,ZZZ10,ZZZ273,ZZZ314,ZZZ981,ZZZER
C
	COMMON /WASPR5/ F1,F2,F11,F12,F22,F21,TES1,TES2,TNEW,VNEW,TOLD,
     *                  VOLD,POLD,IS2
C
	POLD=VI1
	TNEW=VO1
	TOLD=TNEW
	VNEW=VO2
	VOLD=VNEW
C
	IS2=2
C
	TES1=ZZZER
	TES2=ZZZER
C
	CALL WASPR1 (1,VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L,IRET)
C
 1	F1=(-Z2-VI1)
	F2=(-Z1-VI2)
	F11=-Z12
	F12=-Z22
	F21=-Z11
	F22=-Z12
C
	CALL WASPR1 (3,VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L,IRET)
C
	IF (IRET.GT.ZZZ0) GOTO 1
 99	RETURN
	END
C
C================================================================
C
       SUBROUTINE WASPU1 (VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L)
C
C	VI1	P, Pa
C	VI2	U, j/kg
C	VO1	T, deg.C
C	VO2	V, m3/kg
C
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
	COMMON /CONST5/ ZZZ0,ZZZ1,ZZZ2,ZZZ3,ZZZ4,ZZZ5,ZZZ6,ZZZ7,ZZZ8,
     *	ZZZ9,ZZZE2,ZZZE3,ZZZE4,ZZZE5,ZZZE6,ZZZM2,ZZZM3,ZZZM4,ZZZM5,
     *	ZZZM6,ZZZ01,ZZZ10,ZZZ273,ZZZ314,ZZZ981,ZZZER
C
	COMMON /WASPR5/ F1,F2,F11,F12,F22,F21,TES1,TES2,TNEW,VNEW,TOLD,
     *                  VOLD,POLD,IS2
C
C
	POLD=VI1
	TNEW=VO1
	VNEW=VO2
	NS=1
C	
	IS2=7
C
	NX=L
	TES1=ZZZER
	TES2=ZZZER
C
	CALL WASPR1 (1,VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L,IRET)
C
 1	T=(TOLD+ZZZ273)
	F1=(-Z2-VI1)
	F2=(Z-T*Z1)-VI2
	F11=-Z12
	F12=-Z22
	F21=-T*Z11
	F22=(Z2-T*Z12)
C
	CALL WASPR1 (3,VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L,IRET)
C
	IF (IRET.GT.ZZZ0) GOTO 1
 99	RETURN
	END
C
C=================================================================
C
	SUBROUTINE WASHV1 (VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L)
C
C	VI1	H, j/kg
C	VI2	V, m3/kg
C	VO1	T, deg.C
C	VO2	V, m3/kg
C
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
	COMMON /CONST5/ ZZZ0,ZZZ1,ZZZ2,ZZZ3,ZZZ4,ZZZ5,ZZZ6,ZZZ7,ZZZ8,
     *	ZZZ9,ZZZE2,ZZZE3,ZZZE4,ZZZE5,ZZZE6,ZZZM2,ZZZM3,ZZZM4,ZZZM5,
     *	ZZZM6,ZZZ01,ZZZ10,ZZZ273,ZZZ314,ZZZ981,ZZZER
C
	COMMON /WASPR5/ F1,F2,F11,F12,F22,F21,TES1,TES2,TNEW,VNEW,TOLD,
     *                  VOLD,POLD,IS2
C
C
	TNEW=VO1
	VNEW=VI2
	VO2=VI2
	VOLD=VI2
C
	IS2=8
C
	TES1=ZZZER
	TES2=ZZZ1
	F12=ZZZ0
	F21=ZZZ0
	F22=ZZZ1
	F2=ZZZ0
C
	CALL WASPR1 (1,VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L,IRET)
C
 1	T=(TOLD+ZZZ273)
	F1=(Z-T*Z1-VOLD*Z2)-VI1
	F11=-(T*Z11+VOLD*Z12)
C
	CALL WASPR1 (3,VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L,IRET)
C
	IF (IRET.GT.ZZZ0) GOTO 1
C
 99	RETURN
	END
C
C======================================================================
C
      SUBROUTINE WASPR1(IGO,VI1,VI2,VO1,VO2,Z,Z1,Z2,Z11,Z12,Z22,L,IRET)
C
C  
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
	COMMON /CONST5/ ZZZ0,ZZZ1,ZZZ2,ZZZ3,ZZZ4,ZZZ5,ZZZ6,ZZZ7,ZZZ8,
     *	ZZZ9,ZZZE2,ZZZE3,ZZZE4,ZZZE5,ZZZE6,ZZZM2,ZZZM3,ZZZM4,ZZZM5,
     *	ZZZM6,ZZZ01,ZZZ10,ZZZ273,ZZZ314,ZZZ981,ZZZER
C
	COMMON /WASPR5/ F1,F2,F11,F12,F22,F21,TES1,TES2,TNEW,VNEW,TOLD,
     *                  VOLD,POLD,IS2
C
	CHARACTER ALX2(8)*2
C
	DATA ALX2 /'UV','PS','PT','HP','TP','PV','PU','HV'/
C	
C
	GOTO (1,2,3),IGO
C
 1	IF (TNEW.GE.ZZZ0 .AND. VNEW.GT.ZZZ0) GOTO 5
	CALL WASIN1 (VI1,VI2,TNEW,VNEW,L)
 5	TOLD=TNEW
	VOLD=VNEW
 2	CONTINUE
	L=1
	GOTO 260
 3	IRET=0
	DENOM=F11*F22-F12*F21
	ZDELTE=-(F1*F22-F2*F12)/DENOM
	DELV=-(F2*F11-F1*F21)/DENOM
	TNEW=TOLD+ZDELTE
	VNEW=VOLD+DELV
	IF (VNEW.LT.ZZZM3) VNEW=ZZZM3
	DIFF=DABS(F1*ZZZE2/VI1)
	DIF1=DABS(F2*ZZZE2/VI2)
	TOLD=TNEW
	VOLD=VNEW
C
C  CHECK FOR TERMINATION
C
	IF (DIFF.LT.TES1 .AND. DIF1.LT.TES2) GOTO 370
	IF (L.GT.1000) GOTO 4
 260 	CONTINUE
C
	CALL WASR21 (TOLD,VOLD,Z,Z1,Z2,Z11,Z12,Z22)
	L=L+1
	IRET=1
	GOTO 99
 4	WRITE (*,100) L,VI1,VI2,VO1,VO2,TNEW,VNEW
	GOTO 99
 370	CONTINUE
	VO1=TNEW
	VO2=VNEW
  100	FORMAT(/,10X,'NO CONVERGENCE AFTER ',I5,' ITERATIONS',/,
     *           10X,'VI1 =',F14.4,/,
     *           10X,'VI2 =',F14.4,/,
     *	         10X,'VO1 =',F14.6,/,
     *	         10X,'VO2 =',F14.6,/,
     *	         10X,'TNEW=',F14.6,/,
     *	         10X,'VNEW=',F14.6)
C
 99	RETURN
	END
C
C==============================================================
C
      SUBROUTINE WASR21 (TCENT,V,F,DFDT,DFDV,DFDTT,DFDTV,DFDVV)
C
C   HELMHOLTZ FUNCTION AND ITS DERIVATIVES FOR WATER AS A FUNCTION OF
C   TEMPERATURE AND SPECIFIC VOLUME
C   SOURCE : KEENAN, J.H., KEYES, F.G., HILL, P.G., AND MOORE, J.G.,
C   JOHN WILEY AND SONS, NEW YORK, 1969.
C
     	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
 	DIMENSION AC(8),A(10,7),TAJ(7),RAJ(7)
C
	COMMON /CONST5/ ZZZ0,ZZZ1,ZZZ2,ZZZ3,ZZZ4,ZZZ5,ZZZ6,ZZZ7,ZZZ8,
     *	ZZZ9,ZZZE2,ZZZE3,ZZZE4,ZZZE5,ZZZE6,ZZZM2,ZZZM3,ZZZM4,ZZZM5,
     *	ZZZM6,ZZZ01,ZZZ10,ZZZ273,ZZZ314,ZZZ981,ZZZER
C
C     	THE FOLLOWING ARE THE COEFFICIENTS Ci's.
C
        DATA AC/1857.065D0,3229.12D0,-419.465D0,36.6649D0,-20.5516D0,
     *  4.85233D0,46.0D0,-1011.249D0/
C
C	THE FOLLOWING ARE THE COEFFICIENTS Ai's.
C
	DATA A/29.492937D0,-132.13917D0,274.64632D0,-360.93828D0,
     *	342.18431D0,-244.50042D0,155.18535D0,5.9728487D0,-410.30848D0,
     *	-416.05860D0,-5.1985860D0,7.7779182D0,-33.301902D0,
     *  -16.254622D0,-177.31074D0,127.48742D0,137.46153D0,
     *	155.97836D0,337.31180D0,-209.88866D0,6.8335354D0,-26.149751D0,
     *	65.326396D0,-26.181978D0,4*0.0D0,-137.46618D0,-733.96848D0,
     *	-0.156410D0,-0.72546108D0,-9.2734289D0,4.3125840D0,4*0.0D0,
     *	6.7874983D0,10.401717D0,-6.3972405D0,26.409282D0,-47.740374D0,
     *	56.323130D0,4*0.0D0,136.87317D0,645.81880D0,-3.9661401D0,
     *	15.453061D0,-29.142470D0,29.568796D0,4*0.0D0,79.847970D0,
     *	399.17570D0,-0.69048554D0,2.7407416D0,-5.1028070D0,3.9636085D0,
     *	4*0.0D0,13.041253D0,71.531353D0/
C
	DATA R,TC,E/0.46151D0,1.544912D0,4.8D0/
	DATA TAJ/1.544912D0,6*2.5D0/
	DATA RAJ/0.634D0,6*1.0D0/
C
	EPS3=1.0D-10
	T=TCENT+ZZZ273
	VN=V*ZZZE3
	RH=ZZZ1/VN
	TA=ZZZE3/T
	DTA=-TA/T
	D2TA=-ZZZ2*DTA/T
	ALT=DLOG(T)
	ALR=DLOG(RH)
	ER=DEXP(-RH*E)
	TAD=TA-TC
C
	S1=ZZZ0
	S2=ZZZ0
	S3=ZZZ0
C
	DO 2 I=1,6
	XI=I
	TAI=TA**I
	S1=S1+AC(I)*TA/TAI
	S2=S2+AC(I)*(ZZZ1-XI)/TAI
	S3=S3+AC(I)*XI*(XI-ZZZ1)/(TAI*TA)
   2	CONTINUE
C
	P0=S1+ALT*(AC(7)+AC(8)/TA)
	DP0=S2*DTA+AC(7)/T+AC(8)*(ZZZM3-ALT*DTA/(TA*TA))
	D2P0=S3*DTA*DTA+D2TA*S2-AC(7)/(T*T)+AC(8)/(TA*T*T)
C
	S1=ZZZ0
	S2=ZZZ0
	S3=ZZZ0
	S4=ZZZ0
	S5=ZZZ0
	S6=ZZZ0
C
	DO 3 J=1,7
	XJ=J
	TD=TA-TAJ(J)
	RD=RH-RAJ(J)
		IF (DABS(RD).LT.EPS3)  RD=EPS3
	T1=TD**(J-2)
	T2=(J-2)*T1/TD
	T3=(J-3)*T2/TD
C
	S11=ZZZ0
	S12=ZZZ0
	S13=ZZZ0
	B=ZZZ1
C
	DO 4 I=1,8
	XI=I
	GIJ=A(I,J)*B
	DGIJ=GIJ*(XI-ZZZ1)/RD
	D2GIJ=DGIJ*(XI-ZZZ2)/RD
	S11=S11+GIJ
	S12=S12+DGIJ
	S13=S13+D2GIJ
	B=B*RD
  4	CONTINUE
C
	HIJ=A(9,J)+RH*A(10,J)
	S11=S11+ER*HIJ
	S12=S12+ER*(-E*HIJ+A(10,J))
	S13=S13+ER*(E*E*HIJ-ZZZ2*E*A(10,J))
C
	S1=S1+T1*S11
	S2=S2+T2*S11
	S3=S3+T3*S11
	S4=S4+T1*S12
	S5=S5+T1*S13
	S6=S6+T2*S12
C
  3	CONTINUE
C
C  FORMULATION OF Q, FIRST AND SECOND DERIVATIVE OF Q
	Q=TAD*S1
	DQDT=(S1+TAD*S2)*DTA
	DQDTT=(S1+TAD*S2)*D2TA+(ZZZ2*S2+TAD*S3)*DTA*DTA
	DQDR=TAD*S4
	DQDRR=TAD*S5
	DQDTR=DTA*(S4+TAD*S6)
C
C  FORMULATION OF THE HELMHOLTZ FREE ENERGY 'P(T,V)'
C
	P=P0+R*T*(ALR+RH*Q)
C
C  DERIVATIVES OF THE HELMHOLTZ FREE ENERGY
C
	DPDT=DP0+R*(ALR+RH*Q+T*RH*DQDT)
	DPDR=R*T*(ZZZ1/RH+Q+RH*DQDR)
	DPDTT=D2P0+R*RH*(ZZZ2*DQDT+T*DQDTT)
	DPDTR=R*(ZZZ1/RH+Q+RH*DQDR+T*(DQDT+RH*DQDTR))
	DPDRR=R*T*(-ZZZ1/(RH*RH)+ZZZ2*DQDR+RH*DQDRR)
	F=P*ZZZE3
	DFDT=DPDT*ZZZE3
	DFDV=-RH*RH*DPDR*ZZZE6
	DFDTT=DPDTT*ZZZE3
	DFDTV=-RH*RH*DPDTR*ZZZE6
	DFDVV=RH*RH*RH*(RH*DPDRR+ZZZ2*DPDR)*ZZZE3*ZZZE6
C
	RETURN
	END
